library(psych)       #for principal axis factor analysis fa(FieldDataFA, fm="pa", nfactors=3, rotation="varimax")
library(GPArotation) #for principal axis factor analysis
DirectoryToProcess = choose.dir()
DeleteZeros <- TRUE
FilesToProcess = list.files(path = DirectoryToProcess, pattern = "*.txt")   #FileToProcess <- file.choose()
dataFilesCount <- length(FilesToProcess)
cols <- c("CitedBy", "Mendeley", "CiteULike", "Connotea", "Blogs", "News", "Googleplus", "Twitter", "Facebook", "Video", "Wikipedia", "QandA", "F1000")
colsFA <- c("CitedBy", "Mendeley", "Blogs", "News", "Googleplus", "Twitter", "Facebook", "Wikipedia")
VariableHeadings <- c("None", "CitedBy", "Mendeley", "CiteULike", "Connotea", "Blogs", "News", "Googleplus", "Twitter", "Facebook", "Video", "Wikipedia", "F1000") 

dataFile <- 1 # UP to dataFilesCount
FileToProcess <-  paste(DirectoryToProcess,'\\', FilesToProcess[dataFile], sep="")
DataSetName <- substr(sub("_ScopusResultsAltmetricsData.txt","", FilesToProcess[dataFile]),6,100)
FieldData <- read.table(file=FileToProcess, header=TRUE, na.strings=c("", "NA"), quote="\"", sep="\t")
if (DeleteZeros) {
  KeepRows <- rep(FALSE,nrow(FieldData))
  for (i in 1:nrow(FieldData)) {
    for (j in 3:ncol(FieldData)) {
      if (FieldData[i,j] > 0) KeepRows[i] <- TRUE
    }
    if (is.na(FieldData[i,2])) KeepRows[i] <- FALSE
  }
  FieldData <- FieldData[KeepRows,]
} else {
  KeepRows <- rep(TRUE, nrow(FieldData))
  for (i in 1:nrow(FieldData)) {
    if (is.na(FieldData[i,2])) KeepRows[i] <- FALSE
  }
  FieldData <- FieldData[KeepRows,]
}
FieldDataFA <- log(1+FieldData[colsFA])

#Check correlations - remove extra variables if zero correlation
round(cor(FieldDataFA, method = "spearman"), digits = 3)
bartlett.test(FieldDataFA) # p<0.05 is needed
KMO(FieldDataFA) # Less than 0.6 is worrying but at least 0.5 is probably OK
skew(FieldDataFA) #Bigger than 2 is a problem for ML
kurtosi(FieldDataFA) #Bigger than 7 is a problem for ML

#Visually test for number of factors. Saved plot is not very good but is better than nothing.
fa.parallel(FieldDataFA, fa="both", n.iter=100, main=DataSetName)

#[block 2] Set the number of factors here, as manually identified from the information above
nfact <- 3

#[block 3a]Principal axis factor analysis - safe for non-normal
fitPA <- ""
fitPA <- fa(FieldDataFA, fm="pa", nfactors=nfact, rotate="promax", SMC=FALSE) #Avoid varimax because no reason to believe factors uncorrelated. Also reduces cross-factor correlations
print(fitPA, digits=2, cutoff=.3, sort=TRUE) #.3 is a reasonable cut-off 
fa.diagram(fitPA, simple=FALSE, main=paste("P. Axis:", DataSetName))

#[block 3a]Maximum likelihood factor analysis
fitML <- ""
fitML <- fa(FieldDataFA, fm="ml", nfactors=nfact, rotate="promax") #Avoid varimax because no reason to believe factors uncorrelated
print(fitML, digits=2, cutoff=.3, sort=TRUE) 
fa.diagram(fitML, simple=FALSE, main=paste("Max. L.:", DataSetName))

CommentString <- 'Prefer PA due to high skew and kurtosis'

#[block 4]If happy, save the results - also save
sink(file="D:/Altmetric/factorresults2.txt", append=TRUE)
cat("\n**********",  FilesToProcessShortName[dataFile], DirectoryToProcess,"n =",nrow(FieldDataFA) , "**********\n")
cat("Delete zeros:", DeleteZeros, "\n")
print(cor(FieldDataFA, method = "spearman"))
print(fitPA, digits=2, cutoff=.3, sort=TRUE)
print(fitML, digits=2, cutoff=.3, sort=TRUE)
cat(CommentString, "\n")
sink()
tiff(filename = paste(DirectoryToProcess, '\\', DataSetName,'Diag.tif',sep=''),width = 1280, height = 1280, units = "px", res = 800, pointsize = 2, compression = "lzw")
fa.diagram(fitPA, simple=FALSE, main=paste("P. Axis:", DataSetName))
dev.off()
tiff(filename = paste(DirectoryToProcess, '\\', DataSetName,'.tif',sep=''),width = 1280, height = 1280, units = "px", res = 800, pointsize = 2, compression = "lzw")
fa.parallel(FieldDataFA, fa="both", n.iter=100, main=DataSetName)
dev.off()


