library(broom) #For easier access to model fitting data
library(stringr)
DirectoryToProcess = choose.dir()
DeleteZeros <- TRUE
IncludeMendeleyInRegression <- TRUE
FilesToProcess = list.files(path = DirectoryToProcess, pattern = "*.txt")   #FileToProcess <- file.choose()
dataFilesCount <- length(FilesToProcess)
if (IncludeMendeleyInRegression) {
  cols <- c("CitedBy", "Mendeley", "CiteULike", "Connotea", "Blogs", "News", "Googleplus", "Twitter", "Facebook", "Video", "Wikipedia", "QandA", "F1000")
  independantVars <- c("(Intercept)", "Mendeley", "CiteULike", "Connotea", "Blogs", "News", "Googleplus", "Twitter", "Facebook", "Video", "Wikipedia", "F1000") 
} else {
  cols <- c("CitedBy", "CiteULike", "Connotea", "Blogs", "News", "Googleplus", "Twitter", "Facebook", "Video", "Wikipedia", "QandA", "F1000")
  independantVars <- c("(Intercept)", "CiteULike", "Connotea", "Blogs", "News", "Googleplus", "Twitter", "Facebook", "Video", "Wikipedia", "F1000") 
}
VariableHeadings <- c("None", "CitedBy", "Mendeley", "CiteULike", "Connotea", "Blogs", "News", "Googleplus", "Twitter", "Facebook", "Video", "Wikipedia", "F1000") 
length(independantVars)
fieldPValue <- mat.or.vec(dataFilesCount, length(independantVars))
fieldEstimate <- mat.or.vec(dataFilesCount, length(independantVars))
fieldStdError <- mat.or.vec(dataFilesCount, length(independantVars))
fieldStatistic <- mat.or.vec(dataFilesCount, length(independantVars))
fieldAverage <- mat.or.vec(dataFilesCount, length(VariableHeadings))
fieldProportionNonzero <- mat.or.vec(dataFilesCount, length(VariableHeadings))
fieldCorrelation <- mat.or.vec(dataFilesCount, length(VariableHeadings))
fieldSampleSize <- c(dataFilesCount)
fieldRSquared <- c(dataFilesCount)
fieldFitF <- c(dataFilesCount)
fieldFitP <- c(dataFilesCount)
for (dataFile in 1:dataFilesCount) {
  for (altmetric in 1:length(independantVars)) {
    fieldPValue[dataFile,altmetric] <- NA
    fieldEstimate[dataFile,altmetric] <- NA
    fieldStdError[dataFile,altmetric] <- NA
    fieldStatistic[dataFile,altmetric] <- NA
  }
}

RemoveArticlesWithAllZeroAltmetricScores <- function () { #fails
  KeepRows <- rep(FALSE,nrow(FieldData))
  for (i in 1:nrow(FieldData)) {
    for (j in 3:length(independantVars)) {
      if (FieldData[i,j] > 0) KeepRows[i] <- TRUE
    }
  }
  FieldData <- FieldData[KeepRows,]
}

for (dataFile in 1:dataFilesCount) {
  FileToProcess <-  paste(DirectoryToProcess,'\\', FilesToProcess[dataFile], sep="")
  FieldData <- read.table(file=FileToProcess, header=TRUE, na.strings=c("", "NA"), quote="\"", sep="\t")
  if (DeleteZeros) {
    KeepRows <- rep(FALSE,nrow(FieldData))
    for (i in 1:nrow(FieldData)) {
      for (j in 3:length(independantVars)) {
        if (FieldData[i,j] > 0) KeepRows[i] <- TRUE
      }
      if (is.na(FieldData[i,2])) KeepRows[i] <- FALSE
    }
    FieldData <- FieldData[KeepRows,]
  } else {
    KeepRows <- rep(TRUE, nrow(FieldData))
    for (i in 1:nrow(FieldData)) {
      if (is.na(FieldData[i,2])) KeepRows[i] <- FALSE
    }
    FieldData <- FieldData[KeepRows,]
  }
  fieldSampleSize[dataFile] <- nrow(FieldData)
  FieldData[cols] <- log(1+FieldData[cols])
  'Geomean, proportion non-zero, correlation'
  for (j in 2:length(VariableHeadings)){
    fieldAverage[dataFile,j] <- mean(FieldData[,j])
    fieldProportionNonzero[dataFile,j] <- sum(FieldData[,j] !=0 ) / fieldSampleSize[dataFile]
    fieldCorrelation[dataFile,j] <- cor(FieldData[,2],FieldData[,j], use="complete.obs", method="spearman")
  }
  if (IncludeMendeleyInRegression) {
    fit <- lm(CitedBy ~ Mendeley + CiteULike + Connotea + Blogs + News + Googleplus + Twitter + Facebook + Video + Wikipedia + QandA, data=FieldData)
  } else {
    fit <- lm(CitedBy ~ CiteULike + Connotea + Blogs + News + Googleplus + Twitter + Facebook + Video + Wikipedia + QandA, data=FieldData)
  }
  summary(fit) #https://www.statmethods.net/stats/regression.html
  fieldRSquared[dataFile] <- summary(fit)$r.squared
  fieldFitF[dataFile] <- summary(fit)$fstatistic
  fieldFitP[dataFile] <- pf(summary(fit)$fstatistic[1], summary(fit)$fstatistic[2], summary(fit)$fstatistic[3], lower.tail=FALSE)
  fitCoef <- tidy(fit)
  for (i in 1:nrow(fitCoef)) {
    for (j in 1:length(independantVars)) {
      if (fitCoef[i,1] == independantVars[j]) {
        fieldEstimate[dataFile,j] <- fitCoef[i,2]
        fieldStdError[dataFile,j] <- fitCoef[i,3]
        fieldStatistic[dataFile,j] <- fitCoef[i,4]
        fieldPValue[dataFile,j] <- fitCoef[i,5]
      }
    }
  }
}

PrintResults <- function(Name, Files, Columns, Matrix) {
  cat(Name)
  for (j in 1:length(Columns)) {cat(paste("\t", Columns[j],sep="")) }
  cat("\n")
  for (i in 1:length(Files)) {
    cat(Files[i])
    for (j in 1:length(Columns)) {
      cat(paste("\t", Matrix[i,j],sep=""))
    }
    cat("\n")
  }
  cat("\n")
}
sink(file="D:/Altmetric/RegressionResults.txt", append=TRUE)
cat(DirectoryToProcess, "\n")
cat("Delete zeros:", DeleteZeros, "Include Mendeley:", IncludeMendeleyInRegression, "\n")
FilesToProcessShortName <- str_replace(FilesToProcess, "_ScopusResultsAltmetricsData.txt", "")
PrintResults("Estimate",FilesToProcessShortName,independantVars,fieldEstimate)
PrintResults("PValue",FilesToProcessShortName,independantVars,fieldPValue)
PrintResults("StdError",FilesToProcessShortName,independantVars,fieldStdError)
PrintResults("Statistic",FilesToProcessShortName,independantVars,fieldStatistic)
PrintResults("LogMean",FilesToProcessShortName,VariableHeadings,fieldAverage)
PrintResults("ProportionNonzero",FilesToProcessShortName,VariableHeadings,fieldProportionNonzero)
PrintResults("Correlation",FilesToProcessShortName,VariableHeadings,fieldCorrelation)
cat("File", "SampleSize", "RSquared", "FitP", "FitF", "\n", sep="\t")
for (j in 1:length(FilesToProcessShortName)) {cat(FilesToProcessShortName[j],  fieldSampleSize[j], fieldRSquared[j], fieldFitP[j], fieldFitF[j], "\n", sep="\t") }
sink()
